;Quest'opera e' stata rilasciata sotto la licenza Creative Commons
;Attribuzione-Non commerciale-Condividi allo stesso modo 2.5 Italia.
;
;Per leggere una copia della licenza visita il sito web
;http://creativecommons.org/licenses/by-nc-sa/2.5/it/ o spedisci una
;lettera a Creative Commons, 171 Second Street, Suite 300,
;San Francisco, California, 94105, USA.
;
;
;This work is licensed under the Creative Commons
;Attribution-Noncommercial-Share Alike 2.5 Italy License.
;
;To view a copy of this license, visit
;http://creativecommons.org/licenses/by-nc-sa/2.5/it/ or send a letter
;to Creative Commons, 171 Second Street, Suite 300, San Francisco,
;California, 94105, USA.

; ________________________________________________
; |      Gestione seriale per 877 system         |
; |______________________________________________|
; |   Src File                                   |
; |______________________________________________|
; |   Tipo      : ASM                            |
; |   Versione  : 1                              |
; |   Release   : 0                              |
; |   Autore    : Lain Walter                    |
; |   Data      : 20/11/09                       |
; |______________________________________________|

;*******************************************************
;                          NOTE
;*******************************************************
;Per trasmettere, caricare il dato su TXREG. Il caricamento nel registro
; di trasmissione attiva l'interrupt TXIF (PIR1,4 - maschera TXIE su PIE1,4).
;A trasmissione avvenuta si attiva il flag TRMT (TXSTA,1).
;TXIF e TRMT sono a sola lettura. TXIF si pulisce caricando TXREG,
; mentre TRMT si pulisce attivando TXEN (TXSTA,5) e quindi caricando TXREG.
;
;When setting up an Asynchronous Transmission, follow these steps:
;1. Initialize the SPBRG register for the appropriate baud rate.
;   If a high-speed baud rate is desired, set bit BRGH (Section
;   10.1 "USART Baud Rate Generator (BRG)").
;2. Enable the asynchronous serial port by clearing bit SYNC and
;   setting bit SPEN.
;3. If interrupts are desired, then set enable bit TXIE.
;4. If 9-bit transmission is desired, then set transmit bit TX9.
;5. Enable the transmission by setting bit TXEN, which will also
;   set bit TXIF.
;6. If 9-bit transmission is selected, the ninth bit should be
;   loaded in bit TX9D.
;7. Load data to the TXREG register (starts transmission).
;8. If using interrupts, ensure that GIE and PEIE (bits 7 and 6)
;   of the INTCON register are set.
;
;La ricezione carica RCREG alla ricezione del bit di stop, quindi
; abilita il flag interrupt RCIF (PIR1,5 - maschera RCIE su PIE1,5).
;RCREG e' un FIFO a due livelli, quindi si puo' leggere 2 volte per
; recuperare due bytes. Se il terzo byte e' ricevuto mentre il
; registro e' ancora pieno, il byte e' perso e si attiva il flag
; OERR (RCSTA,1). Finche' non si pulisce il flag, la ricezione e'
; bloccata. Se si riceve un bit di stop 0, si attiva il flag FERR
; (RCSTA,2). In questo caso il dato potrebbe essere errato.
;
;When setting up an Asynchronous Reception, follow these steps:
;1. Initialize the SPBRG register for the appropriate baud rate.
;   If a high-speed baud rate is desired, set bit BRGH (Section
;   10.1 "USART Baud Rate Generator (BRG)").
;2. Enable the asynchronous serial port by clearing bit SYNC and
;   setting bit SPEN.
;3. If interrupts are desired, then set enable bit RCIE.
;4. If 9-bit reception is desired, then set bit RX9.
;5. Enable the reception by setting bit CREN.
;6. Flag bit RCIF will be set when reception is complete and an
;   interrupt will be generated if enable bit RCIE is set.
;7. Read the RCSTA register to get the ninth bit (if enabled) and
;   determine if any error occurred during reception.
;8. Read the 8-bit received data by reading the RCREG register.
;9. If any error occurred, clear the error by clearing enable bit
;   CREN.
;10.If using interrupts, ensure that GIE and PEIE (bits 7 and 6)
;   of the INTCON register are set.

;*******************************************************
;                 DEFINIZIONE PROTOCOLLO
;*******************************************************
;
;Per facilitare le cose dal lato PIC, decido di non ricevere
; piu' di due bytes alla volta.
;
;Ciclo di lettura
; CASO STANDARD
; PC --> CMD (READ)
;                         ACK (READ) <-- PIC
;                       DATA (Dato0) <-- PIC
; PC --> DATA (Dato0)
;                  ACK (Dato valido) <-- PIC
;                       DATA (Dato1) <-- PIC
; COMANDO NON CORRETTO
; PC --> CMD (READ)
;                       ACK (Errore) <-- PIC
; PC --> CMD (READ)
;                         ACK (READ) <-- PIC
;                       DATA (Dato0) <-- PIC
; PC --> DATA (Dato0)
;                  ACK (Dato valido) <-- PIC
;                       DATA (Dato1) <-- PIC
;DATO NON CORRETTO
; PC --> CMD (READ)
;                         ACK (READ) <-- PIC
;                       DATA (Dato0) <-- PIC
; PC --> DATA (Dato0)
;              ACK (Dato NON valido) <-- PIC
;                       DATA (Dato0) <-- PIC
; PC --> DATA (Dato0)
;                  ACK (Dato valido) <-- PIC
;                       DATA (Dato1) <-- PIC
;
;Ciclo di scrittura
; CASO STANDARD
; PC --> CMD (WRITE)
;                        ACK (WRITE) <-- PIC
; PC --> DATA (Indirizzo)
; PC --> DATA (Dato)
;                   DATA (Indirizzo) <-- PIC
;                        DATA (Dato) <-- PIC
; PC --> ACK (Indirizzo valido)
; PC --> ACK (Dato valido)
;                 ACK (scrittura OK) <-- PIC
; COMANDO NON CORRETTO
; PC --> CMD (WRITE)
;                       ACK (Errore) <-- PIC
; PC --> CMD (WRITE)
;                        ACK (WRITE) <-- PIC
; PC --> DATA (Indirizzo)
; PC --> DATA (Dato)
;                   DATA (Indirizzo) <-- PIC
;                        DATA (Dato) <-- PIC
; PC --> ACK (Indirizzo valido)
; PC --> ACK (Dato valido)
;                 ACK (scrittura OK) <-- PIC
;INDIRIZZO NON CORRETTO
; PC --> CMD (WRITE)
;                        ACK (WRITE) <-- PIC
; PC --> DATA (Indirizzo)
; PC --> DATA (Dato)
;                   DATA (Indirizzo) <-- PIC
;                        DATA (Dato) <-- PIC
; PC --> (Indirizzo NON valido)
; PC --> DATA (Indirizzo)
;                   DATA (Indirizzo) <-- PIC
; PC --> ACK (Indirizzo valido)
;                 ACK (scrittura OK) <-- PIC
;DATO NON CORRETTO
; PC --> CMD (WRITE)
;                        ACK (WRITE) <-- PIC
; PC --> DATA (Indirizzo)
; PC --> DATA (Dato)
;                   DATA (Indirizzo) <-- PIC
;                        DATA (Dato) <-- PIC
; PC --> (Dato NON valido)
; PC --> DATA (Dato)
;                   DATA (Dato) <-- PIC
; PC --> ACK (Dato valido)
;                 ACK (scrittura OK) <-- PIC
;BLOCCO NON CORRETTO
; PC --> CMD (WRITE)
;                        ACK (WRITE) <-- PIC
; PC --> DATA (Indirizzo)
; PC --> DATA (Dato)
;                   DATA (Indirizzo) <-- PIC
;                        DATA (Dato) <-- PIC
; PC --> (Blocco errato)
;                ACK (Blocco errato) <-- PIC
; PC --> DATA (Indirizzo)
; PC --> DATA (Dato)
;                   DATA (Indirizzo) <-- PIC
;                        DATA (Dato) <-- PIC
; PC --> ACK (Indirizzo valido)
; PC --> ACK (Dato valido)
;                 ACK (scrittura OK) <-- PIC
; ERRORE SCRITTURA
; PC --> CMD (WRITE)
;                        ACK (WRITE) <-- PIC
; PC --> DATA (Indirizzo)
; PC --> DATA (Dato)
;                   DATA (Indirizzo) <-- PIC
;                        DATA (Dato) <-- PIC
; PC --> ACK (Indirizzo valido)
; PC --> ACK (Dato valido)
;             ACK (scrittura NON OK) <-- PIC
;
;Byte CMD:   7 6 5 4 3 2 1 0 (primi due bit =10)
; READ       1 0 1 0 1 0 0 1
; WRITE      0 1 0 1 0 1 0 1
; PING       1 0 1 0 1 1 0 1
;
;Byte ACK:   7 6 5 4 3 2 1 0 (primi due bit =11)
; Ping       0 0 0 0 0 0 1 1 risposta al ping
; Read       1 0 0 0 0 0 1 1 risposta al comando read
; Write      1 1 0 0 0 0 1 1 risposta al comando write
; End data   1 0 1 0 1 0 1 0 risposta in lettura all'ultimo dato corretto
; Ind. OK    0 0 0 1 1 0 1 1 indirizzo (bit 2,3 = 01)
; Ind. ERR   0 0 1 0 1 0 1 1
; Dato OK    0 0 1 1 0 0 1 1 dato (bit 2,3 = 00)
; Dato ERR   0 1 0 1 0 0 1 1
; Ric. ERR   0 1 0 1 0 1 1 1 procedura (bit 2,3 = 10)
; Wrt. OK    0 1 1 0 0 1 1 1 scrittura su E^2 OK
; Wrt. ERR   1 0 1 0 0 1 1 1 scrittura su E^2 ERRORE ????????

;_______________________________________________________Inizializza seriale
; Routine di init della seriale
;Registri modificati:
;       Sistema
;       w_SYS_CTL
;       b_SER_CTL
;       b_SER_DATA
;       b_SER_ADDR
Ser_Init:
        BANKSEL TXSTA                       ;Bank 1
        MOVLF c_SER_TXSTA,TXSTA             ;imposta modalita' trasmissione seriale
        MOVLF c_SER_SPBRG,SPBRG             ;imposta baud rate
        BANKSEL RCSTA                       ;Bank 0
        MOVLF c_SER_RCSTA,RCSTA             ;imposta modalita' ricezione seriale
        BCF RCSTA,CREN                      ;pulisce il flag errore overrun (OERR)
        MOVF RCREG,w                        ;svuota buffer ricezione
        MOVF RCREG,w                        ;
        MOVF RCREG,w                        ;
        BSF RCSTA,CREN                      ;attiva ricezione
        CLRF b_SER_CTL                      ;clear flags controllo comunicazione
        BCF f_SER_GO                        ;clear flag seriale attiva
        BSF f_SER_OK                        ;set porta inizializzata correttamente
        CLRF b_SER_DATA                     ;svuota buffer dato
        CLRF b_SER_ADDR                     ;svuota buffer indirizzo
        RETURN

;_______________________________________________________Comunicazione seriale
; Questa enorme sub gestisce tutta la comunicazione seriale e l'I/O sulla EEPROM
;
;Registri modificati:
;       Sistema
;       R0H
;       R0L
;       w_SYS_CTL
;       b_SER_CTL
;       b_SER_DATA
;       b_SER_ADDR
Ser_Comm:
        BANKSEL PIR1                        ;Bank 0
        BTFSS PIR1,RCIF                     ;controlla se ricevuto dati
        RETURN                              ; no
        BSF f_SER_GO                        ; si, attiva flag seriale attiva
        BTFSC RCSTA,FERR                    ;controlla se frame error
        GOTO SER_CER                        ; errore - verificare perche'???
        BTFSC RCSTA,OERR                    ;controlla se overrun - verificare perche'???
        GOTO SER_CER                        ; errore comunicazione
        MOVFF RCREG,b_SER_BUFF              ;legge primo byte
;decodifica byte e gestione
        BTFSC f_SER_READ                    ;modalita' lettura?
        GOTO SER_CR1                        ; si
        BTFSC f_SER_WRITE                   ; no, modalita' scrittura?
        GOTO SER_CW1                        ;  si
        MOVF b_SER_BUFF,w                   ;  no, stato iniziale, carica byte
        XORLW c_SER_CMDR                    ;verifica se comando lettura
        BZ SER_CR0                          ; si
        MOVF b_SER_BUFF,w                   ; no, ricarica byte
        XORLW c_SER_CMDW                    ; verifica se comando scrittura
        BZ SER_CW0                          ;  si
        MOVF b_SER_BUFF,w                   ; no, ricarica byte
        XORLW c_SER_CMDWEXT                 ; verifica se comando scrittura estesa
        BZ SER_CX0                          ;  si
        MOVF b_SER_BUFF,w                   ;  no, ricarica byte
        XORLW c_SER_CMDP                    ;    verifica se comando ping
        BZ SER_CP0                          ;     si
SER_CER BCF RCSTA,CREN                      ;errore ricezione, pulisce il flag OERR
        BANKSEL TXSTA                       ;Bank 1
        BSF TXSTA,TXEN                      ; attiva trasmissione
        BANKSEL TXREG                       ;Bank 0
        MOVLF c_SER_ACKRE,TXREG             ;invia errore ricezione
        CLRF b_SER_CTL                      ;pulisce flags
        MOVF RCREG,w                        ;svuota buffer ricezione
        MOVF RCREG,w                        ;
        MOVF RCREG,w                        ;
        BSF RCSTA,CREN                      ;riattiva ricezione
        RETURN
SER_CR0 BCF f_SER_OK                        ;disattiva porta OK
        BANKSEL TXSTA                       ;Bank 1
        BSF TXSTA,TXEN                      ;Attiva trasmissione
        BANKSEL TXREG                       ;Bank 0
        MOVLF c_SER_ACKR,TXREG              ;invia ack comando lettura
        BCF RCSTA,CREN                      ;pulisce il flag errore overrun (OERR)
        MOVF RCREG,w                        ;svuota buffer ricezione
        MOVF RCREG,w                        ;
        MOVF RCREG,w                        ;
        BSF RCSTA,CREN                      ;riattiva ricezione
        BSF f_SER_GO                        ;attiva flag comunicazione in corso
        CLRF b_SER_CTL                      ;pulisce flags
        BSF f_SER_READ                      ;attiva stato lettura
        BSF f_SER_STEP1                     ;attiva step 1
        BSF f_SER_WAIT1                     ; attiva attesa reset seriale
        BSF f_SER_WAIT2                     ; attiva attesa reset seriale
        CLRF b_SER_ADDR                     ;azzera indirizzo EEPROM
        GOTO SER_CR3                        ;invia primo dato
SER_CR1 BCF RCSTA,CREN                      ;pulisce il flag errore overrun (OERR)
        MOVF RCREG,w                        ;svuota buffer ricezione
        MOVF RCREG,w                        ;
        MOVF RCREG,w                        ;
        BSF RCSTA,CREN                      ;riattiva ricezione
        MOVF b_SER_BUFF,w                   ; legge primo byte
        XORWF b_SER_DATA,w                  ;controlla se dato corretto
        BNZ SER_CR2                         ; no
        BANKSEL TXSTA                       ;Bank 1
        BSF TXSTA,TXEN                      ; si, attiva trasmissione
        BANKSEL TXREG                       ;Bank 0
        MOVLW c_SER_ACKDO                   ;carica ack dato corretto
        BTFSC f_SER_LAST                    ;verifica se ultimo dato
        MOVLW c_SER_ACKEND                  ; si, carica ack fine dati
        MOVWF TXREG                         ;invia ack
        BSF f_SER_WAIT1                     ; attiva attesa reset seriale
        BSF f_SER_WAIT2                     ; attiva attesa reset seriale
        GOTO SER_CR3                        ;invia dato successivo
SER_CR2 NOP
        BANKSEL TXSTA                       ;Bank 1
        BSF TXSTA,TXEN                      ;Attiva trasmissione
        BANKSEL TXREG                       ;Bank 0
        MOVLF c_SER_ACKDE,TXREG             ;invia ack dato NON corretto
        BANKSEL TXSTA                       ;Bank 1
        BTFSS TXSTA,TRMT                    ;aspetta fine trasmissione
        GOTO $-1                            ;
        BSF TXSTA,TXEN                      ;Attiva trasmissione
        BANKSEL TXREG                       ;Bank 0
        MOVFF b_SER_DATA,TXREG              ;re-invia dato
        BSF f_SER_WAIT1                     ; attiva attesa reset seriale
        BSF f_SER_WAIT2                     ; attiva attesa reset seriale
        RETURN
SER_CR3 BTFSS f_SER_LAST                    ;verifica se deve proseguire
        GOTO SER_CR4                        ; si
        CLRF b_SER_CTL                      ; no, fine
        BCF f_SER_GO                        ;disattiva flag seriale attiva
        BSF f_SER_OK                        ;porta OK
        RETURN
SER_CR4 BSF f_SER_WAIT1                     ; attiva attesa reset seriale
        BSF f_SER_WAIT2                     ; attiva attesa reset seriale
        MOVF b_SER_ADDR,w                   ;carica indirizzo da leggere
        CALL E2_Read                        ;legge dato EEPROM
        MOVWF b_SER_DATA                    ;salva dato letto
        BANKSEL TXSTA                       ;Bank 1
        BSF TXSTA,TXEN                      ;Attiva trasmissione
        BANKSEL TXREG                       ;Bank 0
        MOVWF TXREG                         ;invia dato
        CLRF b_SER_CTL                      ;pulisce flags
        INCFSZ b_SER_ADDR,f                 ; prossimo dato
        GOTO SER_CR5                        ; ha finito i dati?
        MOVLF 0xFF,b_SER_ADDR               ; si, fine trasmissione
        BSF f_SER_LAST                      ;
SER_CR5 BSF f_SER_READ                      ; no, riattiva stato lettura
        BSF f_SER_STEP1                     ; attiva step 1
        BSF f_SER_WAIT1                     ; attiva attesa reset seriale
        BSF f_SER_WAIT2                     ; attiva attesa reset seriale
        RETURN

SER_CW0 BCF f_SER_OK                        ;disattiva porta OK
        BANKSEL TXSTA                       ;Bank 1
        BSF TXSTA,TXEN                      ;Attiva trasmissione
        BANKSEL TXREG                       ;Bank 0
        MOVLF c_SER_ACKW,TXREG              ;invia ack comando scrittura
        BSF f_SER_GO                        ;attiva flag comunicazione in corso
        CLRF b_SER_CTL                      ;pulisce flags
        BSF f_SER_WRITE                     ;attiva stato scrittura
        BSF f_SER_STEP1                     ;attiva step 1
        BSF f_SER_WAIT1                     ; attiva attesa reset seriale
        BSF f_SER_WAIT2                     ; attiva attesa reset seriale
        BCF RCSTA,CREN                      ;pulisce il flag errore overrun (OERR)
        MOVF RCREG,w                        ;svuota buffer ricezione
        MOVF RCREG,w                        ;
        MOVF RCREG,w                        ;
        BSF RCSTA,CREN                      ;riattiva ricezione
        RETURN
SER_CW1 BTFSS f_SER_STEP1                   ;step 1?
        GOTO SER_CW2                        ; no
        MOVFF b_SER_BUFF,b_SER_ADDR         ; si, carica indirizzo
        BTFSC RCSTA,OERR                    ;controlla se overrun - verificare perche'???
        GOTO SER_CER                        ; errore comunicazione
        BTFSC RCSTA,FERR                    ;controlla se frame error
        GOTO SER_CER                        ; errore - verificare perche'???
        MOVFF RCREG,b_SER_BUFF              ; legge secondo byte
        BCF RCSTA,CREN                      ;pulisce il flag errore overrun (OERR)
        MOVF RCREG,w                        ;svuota buffer ricezione
        MOVF RCREG,w                        ;
        MOVF RCREG,w                        ;
        BSF RCSTA,CREN                      ;riattiva ricezione
        MOVFF b_SER_BUFF,b_SER_DATA         ;
        BANKSEL TXSTA                       ;Bank 1
        BSF TXSTA,TXEN                      ;Attiva trasmissione
        BANKSEL TXREG                       ;Bank 0
        MOVFF b_SER_ADDR,TXREG              ;re-invia indirizzo
        BANKSEL TXSTA                       ;Bank 1
        BTFSS TXSTA,TRMT                    ;aspetta fine trasmissione
        GOTO $-1                            ;
        BSF TXSTA,TXEN                      ;Attiva trasmissione
        BANKSEL TXREG                       ;Bank 0
        MOVFF b_SER_DATA,TXREG              ;re-invia dato
        BSF f_SER_WAIT1                     ;attiva attesa reset seriale
        BSF f_SER_WAIT2                     ;attiva attesa reset seriale
        BCF f_SER_STEP1                     ;disattiva step 1
        BSF f_SER_STEP2                     ;attiva step 2
        RETURN
SER_CW2 BTFSS f_SER_STEP2                   ;step 2?
        GOTO SER_CW5                        ; no
        MOVF b_SER_BUFF,w                   ; si, verifica dato ricevuto
        XORLW c_SER_ACKAO                   ;indirizzo OK?
        BZ SER_CW7                          ; si
        MOVF b_SER_BUFF,w                   ; no
        XORLW c_SER_ACKAE                   ;indirizzo err?
        BZ SER_CW3                          ; si
        MOVF b_SER_BUFF,w                   ; no
        XORLW c_SER_ACKDE                   ;dato ERR?
        BZ SER_CW4                          ; si
        MOVF b_SER_BUFF,w                   ; no
        XORLW c_SER_ACKBE                   ;blocco ERR?
        BNZ SER_CER                         ; no, c'e' qualcosa che non va
        BCF RCSTA,CREN                      ;pulisce il flag errore overrun (OERR)
        MOVF RCREG,w                        ;svuota buffer ricezione
        MOVF RCREG,w                        ;
        MOVF RCREG,w                        ;
        BSF RCSTA,CREN                      ;riattiva ricezione
        BANKSEL TXSTA                       ;Bank 1
        BSF TXSTA,TXEN                      ;Attiva trasmissione
        BANKSEL TXREG                       ;Bank 0
        MOVLF c_SER_ACKBE,TXREG             ;invia ack errore blocco
        BSF f_SER_STEP1                     ;riattiva step 1
        BCF f_SER_STEP2                     ;disattiva step 2
        BSF f_SER_WAIT1                     ;attiva attesa reset seriale
        BSF f_SER_WAIT2                     ;attiva attesa reset seriale
        RETURN
SER_CW3 BTFSC RCSTA,OERR                    ;controlla se overrun - verificare perche'???
        GOTO SER_CER                        ; errore comunicazione
        BTFSC RCSTA,FERR                    ;controlla se frame error
        GOTO SER_CER                        ; errore - verificare perche'???
        MOVFF RCREG,b_SER_BUFF              ;indirizzo sbagliato, ricarica indirizzo
        BCF RCSTA,CREN                      ;pulisce il flag errore overrun (OERR)
        MOVF RCREG,w                        ;svuota buffer ricezione
        MOVF RCREG,w                        ;
        MOVF RCREG,w                        ;
        BSF RCSTA,CREN                      ;riattiva ricezione
        MOVFF b_SER_BUFF,b_SER_ADDR         ;
        BANKSEL TXSTA                       ;Bank 1
        BSF TXSTA,TXEN                      ;Attiva trasmissione
        BANKSEL TXREG                       ;Bank 0
        MOVFF b_SER_ADDR,TXREG              ;re-invia indirizzo
        BCF f_SER_STEP2                     ;disattiva step 2
        BSF f_SER_STEP3                     ;attiva step 3 (attesa ack indirizzo)
        BSF f_SER_WAIT1                     ;attiva attesa reset seriale
        BSF f_SER_WAIT2                     ;attiva attesa reset seriale
        RETURN
SER_CW4 BTFSC RCSTA,OERR                    ;controlla se overrun - verificare perche'???
        GOTO SER_CER                        ; errore comunicazione
        BTFSC RCSTA,FERR                    ;controlla se frame error
        GOTO SER_CER                        ; errore - verificare perche'???
        MOVFF RCREG,b_SER_BUFF              ;dato sbagliato, ricarica dato
        BCF RCSTA,CREN                      ;pulisce il flag errore overrun (OERR)
        MOVF RCREG,w                        ;svuota buffer ricezione
        MOVF RCREG,w                        ;
        MOVF RCREG,w                        ;
        BSF RCSTA,CREN                      ;riattiva ricezione
        MOVFF b_SER_BUFF,b_SER_DATA         ;
        BANKSEL TXSTA                       ;Bank 1
        BSF TXSTA,TXEN                      ;Attiva trasmissione
        BANKSEL TXREG                       ;Bank 0
        MOVFF b_SER_DATA,TXREG              ;re-invia dato
        BCF f_SER_STEP2                     ;disattiva step 2
        BSF f_SER_STEP4                     ;attiva step 4 (attesa ack dato)
        BSF f_SER_WAIT1                     ;attiva attesa reset seriale
        BSF f_SER_WAIT2                     ;attiva attesa reset seriale
        RETURN
SER_CW5 BTFSS f_SER_STEP3                   ;step 3?
        GOTO SER_CW6                        ; no
        MOVF b_SER_BUFF,w                   ;
        XORLW c_SER_ACKAO                   ;indirizzo OK?
        BZ SER_CW7                          ; si
        MOVF b_SER_BUFF,w                   ; no
        XORLW c_SER_ACKAE                   ;indirizzo err?
        BZ SER_CW3                          ; si
        GOTO SER_CER                        ; no, non dovrei essere qui, errore
        GOTO SER_CW7                        ;blocco OK, scrive
SER_CW6 BTFSS f_SER_STEP4                   ;step 4?
        GOTO SER_CER                        ; no, non dovrei essere qui, errore
        XORLW c_SER_ACKDO                   ;dato OK?
        BZ SER_CW7                          ; si
        MOVF b_SER_BUFF,w                   ; no
        XORLW c_SER_ACKDE                   ;dato err?
        BZ SER_CW4                          ; si
        GOTO SER_CER                        ; no, non dovrei essere qui, errore
        GOTO SER_CW7                        ;blocco OK, scrive
SER_CW7 BTFSC RCSTA,OERR                    ;controlla se overrun - verificare perche'???
        GOTO SER_CER                        ; errore comunicazione
        BTFSC RCSTA,FERR                    ;controlla se frame error
        GOTO SER_CER                        ; errore - verificare perche'???
        MOVFF RCREG,b_SER_BUFF              ;legge secondo byte
        BCF RCSTA,CREN                      ;pulisce il flag errore overrun (OERR)
        MOVF RCREG,w                        ;svuota buffer ricezione
        MOVF RCREG,w                        ;
        MOVF RCREG,w                        ;
        BSF RCSTA,CREN                      ;riattiva ricezione
        MOVF b_SER_BUFF,w                   ;dato OK?
        XORLW c_SER_ACKDO                   ;
        BNZ SER_CER                         ; no, c'e' qualcosa che non va
        BSF f_SER_WAIT1                     ; si, attiva attesa reset seriale
        BSF f_SER_WAIT2                     ;
        CALL E2_Write                       ;scrive dato EEPROM
        BANKSEL TXSTA                       ;Bank 1
        BSF TXSTA,TXEN                      ;Attiva trasmissione
        BANKSEL TXREG                       ;Bank 0
        MOVLF c_SER_ACKWO,TXREG             ;invia ack scrittura OK
        CLRF b_SER_CTL                      ;pulisce flags
        BCF f_SER_GO                        ;fine comunicazione
        BSF f_SER_OK                        ;porta OK
        RETURN

SER_CX0 NOP
        BANKSEL TXSTA                       ;Bank 1
        BSF TXSTA,TXEN                      ;Attiva trasmissione
        BANKSEL TXREG                       ;Bank 0
        MOVLF c_SER_ACKNOEXT,TXREG          ;invia ack scrittura estesa non disponibile
        CLRF b_SER_CTL                      ;pulisce flags
        BCF RCSTA,CREN                      ;pulisce il flag errore overrun (OERR)
        MOVF RCREG,w                        ;svuota buffer ricezione
        MOVF RCREG,w                        ;
        MOVF RCREG,w                        ;
        BSF RCSTA,CREN                      ;riattiva ricezione
        RETURN

SER_CP0 NOP
        BANKSEL TXSTA                       ;Bank 1
        BSF TXSTA,TXEN                      ;Attiva trasmissione
        BANKSEL TXREG                       ;Bank 0
        MOVLF c_SER_ACKP,TXREG              ;invia ack comando ping
        CLRF b_SER_CTL                      ;pulisce flags
        BCF RCSTA,CREN                      ;pulisce il flag errore overrun (OERR)
        MOVF RCREG,w                        ;svuota buffer ricezione
        MOVF RCREG,w                        ;
        MOVF RCREG,w                        ;
        BSF RCSTA,CREN                      ;riattiva ricezione
        RETURN

;_______________________________________________________Reset seriale
; Questa sub resetta la comunicazione seriale se in timeout
;
;Registri modificati:
;       Sistema
;       w_SYS_CTL
;       b_SER_CTL
;       b_SER_DATA
;       b_SER_ADDR
Ser_Reset:
        BTFSC f_SER_OK                      ;verifica se porta OK
        RETURN                              ; si, fine
        BTFSS f_SER_WAIT1                   ; no, reset seriale in caso di blocco
        GOTO SER_R01                        ;
        BTFSS f_SER_WAIT2                   ;attesa 1
        BCF f_SER_WAIT1                     ;attesa 2
        BCF f_SER_WAIT2                     ;
        RETURN
SER_R01 CALL Ser_Init                       ;reimposta porta seriale
        RETURN

;_______________________________________________________Lettura dato EEPROM
; Lettura dalla memoria E^2.
;Dati in ingresso
;       WREG = indirizzo da leggere
;Dati in uscita
;       WREG = dato
;Registri modificati:
;       Sistema
E2_Read:
        BSF STATUS,RP1                      ;Bank 2
        BCF STATUS,RP0                      ;
        MOVWF EEADR                         ;Carica indirizzo EEPROM
        BSF STATUS,RP0                      ;Bank 3 memoria
        BCF EECON1,EEPGD                    ;Point to Data memory
        BSF EECON1,RD                       ;Lettura
        BCF STATUS,RP0                      ;Bank 2
        MOVF EEDATA,w                       ;W = EEDATA
        BCF STATUS,RP1                      ;Bank 0
        RETURN

;_______________________________________________________Scrittura dato EEPROM
; Scrittura sulla memoria E^2.
;Dati in ingresso
; b_SER_DATA e b_SER_ADDR
;Dati in uscita
;
;Registri modificati:
;        Sistema
E2_Write:
        BSF STATUS,RP1                      ;Bank 3
        BSF STATUS,RP0                      ;
        BTFSC EECON1,WR                     ;Wait for write
        GOTO $-1                            ;to complete
        BANKSEL b_SER_ADDR                  ;Bank 0
        MOVF b_SER_ADDR,w                   ;Carica indirizzo
        BANKSEL EEADR                       ;Bank 2
        MOVWF EEADR                         ;Address to write
        BANKSEL b_SER_ADDR                  ;Bank 0
        MOVF b_SER_DATA,w                   ;Carica dato
        BANKSEL EEDATA                      ;Bank 2
        MOVWF EEDATA                        ;to write
        BSF STATUS,RP0                      ;Bank 3
        BCF EECON1,EEPGD                    ;Point to DATA memory
        BSF EECON1,WREN                     ;Enable writes
        BCF INTCON,GIE                      ;Disable INTs.
        MOVLW 0x55                          ;
        MOVWF EECON2                        ;Write 55h
        MOVLW 0xAA                          ;
        MOVWF EECON2                        ;Write AAh
        BSF EECON1,WR                       ;Set WR bit to begin write
        BSF INTCON,GIE                      ;Enable INTs.
        BCF EECON1,WREN                     ;Disable writes
        BANKSEL b_SER_ADDR                  ;Bank 0
        RETURN
